
class Apollo extends AIController {

  towns_used = null


  function Start();
  function Stop();

  constructor() {
     this.towns_used = AIList();
  }
}


function Apollo::GetHighestPopulationTown()
{
  /* Create list of towns */
  local town_list = AITownList();
  /* Remove the towns we already use */
  town_list.RemoveList(this.towns_used);

  /* Sort towns on population and keep the highest one. Then return it */
  town_list.Valuate(AITown.GetPopulation);
  town_list.KeepTop(1);
  local town = town_list.Begin();
  return town;
}

function Apollo::PlaceSign(town)
{
  /* determine the X and Y coordinates of the town center */
  local town_center = AITown.GetLocation(town);
  local town_x = AIMap.GetTileX(town_center);
  local town_y = AIMap.GetTileY(town_center);

  /* forge the sign string */
  local sign_text = "(X:" + town_x + ", Y:" + town_y + ")";

  /*send it to the logs */
  AILog.Info(sign_text);

  /*then build the sign*/
  AISign.BuildSign(town_center, sign_text);
}


function Apollo::CheckPossibleRoutes()
{
  /* Find the town with the most inhabitants */
  local town = this.GetHighestPopulationTown();

  /* read out its name and pop and log it */
  local town_name = AITown.GetName(town);
  local town_pop = AITown.GetPopulation(town);
  AILog.Info("Best place to start a bus connection would be " + town_name + " with " + town_pop + " inhabitants");

  /* town needs at least 750 inhabitants to be profitable */
  if(town_pop > 750)
  {
     AILog.Info("We're gonna build some bus stations there");

     /* actually we're not building bus stops yet, we just place a sign */
     this.PlaceSign(town);

     /* add town to the used list */
     this.towns_used.AddItem(town, 0);
  }
  else
  {
     AILog.Info("Town is too small");
  }
}

function Apollo::Start()
{
  /* Sleep 1 tick, as we can't execute anything in tick 0 */
  Sleep(1);

  /* Give the boy a name */
  if (!AICompany.SetCompanyName("Apollo")) {
    local i = 2;
    while (!AICompany.SetCompanyName("Apollo #" + i)) {
      i++;
    }
  }
  /* Say hello to the user */
  AILog.Info("I am Apollo. I will create busses until I make $1 million. If I reach that, I will reward you by making even more busses.");

  /* We need our local ticker, as GetTick() will skip ticks */
  local ticker = 0;
  /* Determine time we may sleep */
  local sleepingtime = 10;

  /* Let's go on for ever */
  while (true) {
    /* Let the outside world know we're alive */
    AILog.Info("Apollo lives at tick"+ticker);
    /* check for new routes to waste money on */
    this.CheckPossibleRoutes();
    /* Make sure we do not create infinite loops */
    Sleep(sleepingtime);
    ticker += sleepingtime;
  }
}

function Apollo::Stop()
{
}